-- @ PeterJ - euroDZN
-- @ 09/09/2013
--
-- http://eurodzn.wordpress.com/
--
-- Copyright (C) euroDZN, Confidential, All Rights Reserved.

fertSpreader = {};

function fertSpreader.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Sprayer, specializations);
end;

function fertSpreader:load(xmlFile)

	self.setExtraBody = SpecializationUtil.callSpecializationsFunction("setExtraBody");
	self.extraBodyNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fertSpreader#extraBody"));
	self.lowBodyPlane = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fertSpreader#lowBodyPlane"));
	if self.extraBodyNode ~= nil then
		self.extraCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fertSpreader#extraCapacity"), 0);
	end;
	self.extraBodyOn = false;
	
end;

function fertSpreader:delete()
end;

function fertSpreader:readStream(streamId, connection)
	local extraBodyOn = streamReadBool(streamId);
	self:setExtraBody(extraBodyOn, true);
end;

function fertSpreader:writeStream(streamId, connection)
	streamWriteBool(streamId, self.extraBodyOn);
end;

function fertSpreader:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local extraBodyOn = getXMLBool(xmlFile, key.."#extraBodyOn");
	if extraBodyOn ~= nil then
		self:setExtraBody(extraBodyOn);
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function fertSpreader:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "extraBodyOn='"..tostring(self.extraBodyOn).."'";
	return attributes, nil;
end;

function fertSpreader:mouseEvent(posX, posY, isDown, isUp, button)
end;

function fertSpreader:keyEvent(unicode, sym, modifier, isDown)
end;

function fertSpreader:update(dt)
	if self.isClient then
		if self.fillLevel <= 0 and self:getIsActiveForInput() then
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
				self:setExtraBody(not self.extraBodyOn);
			end;
		end;
	end;
end;

function fertSpreader:updateTick(dt)
	if not self:getIsActive() then
		if self.isTurnedOn then
			self:setIsTurnedOn(false);
		end;
	end;
end;

function fertSpreader:draw()
	if self.isClient then
		if self.fillLevel <= 0 then
			if self.extraBodyOn then
				g_currentMission:addHelpButtonText(g_i18n:getText("RemoveBody"), InputBinding.IMPLEMENT_EXTRA3);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("AddBody"), InputBinding.IMPLEMENT_EXTRA3);
			end;
		end;
	end;
end;

function fertSpreader:setExtraBody(extraBodyOn,noEventSend)
	if extraBodyOn ~= self.extraBodyOn then
		SetExtraBodyEvent.sendEvent(self, extraBodyOn, noEventSend)
  
		self.extraBodyOn = extraBodyOn;
		if self.isClient then
			if self.extraBodyNode ~= nil then
				if self.extraBodyOn then
					setVisibility(self.extraBodyNode, true);
					setVisibility(self.lowBodyPlane, false);
					self.capacity = self.capacity + self.extraCapacity;
				else
					setVisibility(self.extraBodyNode, false);
					setVisibility(self.lowBodyPlane, true);
					self.capacity = self.capacity - self.extraCapacity;
				end;
			end;
		end;
	end;
end;

SetExtraBodyEvent = {};
SetExtraBodyEvent_mt = Class(SetExtraBodyEvent, Event);

InitEventClass(SetExtraBodyEvent, "SetExtraBodyEvent");

function SetExtraBodyEvent:emptyNew()
	local self = Event:new(SetExtraBodyEvent_mt);
	return self;
end;

function SetExtraBodyEvent:new(object, extraBodyOn)
	local self = SetExtraBodyEvent:emptyNew()
	self.object = object;
	self.extraBodyOn = extraBodyOn;
	return self;
end;

function SetExtraBodyEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.extraBodyOn = streamReadBool(streamId);
	self.object = networkGetObject(id);
	self:run(connection);
end;

function SetExtraBodyEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.extraBodyOn);
end;

function SetExtraBodyEvent:run(connection)
	if not connection:getIsServer() then
	g_server:broadcastEvent(self, false, connection, self.object);
end;
if self.object ~= nil then
	self.object:setExtraBody(self.extraBodyOn, true);
	end;
end;

function SetExtraBodyEvent.sendEvent(vehicle, extraBodyOn, noEventSend)
	if extraBodyOn ~= vehicle.extraBodyOn then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(SetExtraBodyEvent:new(vehicle, extraBodyOn), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(SetExtraBodyEvent:new(vehicle, extraBodyOn));
			end;
		end;
	end;
end;